#ifndef _H_qrman
#define _H_qrman

#ifndef RI_H
#include <ri/ri.h>
#endif

/*
 * picking callback
 */
typedef struct {
	RtInt	tagCount;	/* number of elements in tagList */
	RtInt	*tagList;	/* list of picktags */
	RtFloat	z;
} RtPickCallback;

RtVoid QRMSetPickCallback(RtVoid (*func)(RtPickCallback *));

/*
 * info callback
 */
typedef struct {
	RtFloat bound[6]; /* rendered camera space bounds */
} RtInfoCallback;

RtVoid QRMSetInfoCallback(RtVoid (*func)(RtInfoCallback *));

/*
 * RIB reader intercept
 */
typedef struct RIBHandlers {
	RtToken (*AreaLightSourceV)(char *, RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*AtmosphereV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*AttributeBegin)(void);
	RtVoid (*AttributeEnd)(void);
	RtVoid (*AttributeV)(RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*Basis)(RtBasis, RtInt, RtBasis, RtInt);
	RtVoid (*Bound)(RtBound);
	RtVoid (*CircleV)(RtFloat, RtFloat, RtInt, RtToken *, RtPointer *);
	RtVoid (*Clipping)(RtFloat, RtFloat);
	RtVoid (*Color)(RtColor);
	RtVoid (*ColorSamples)(RtInt, RtFloat *, RtFloat *);
	RtVoid (*ConcatTransform)(RtMatrix);
	RtVoid (*ConeV)(RtFloat, RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtVoid (*CoordSysTransform)(RtToken);
	RtToken (*CoordinateSystem)(char *);
	RtToken (*CreateHandle)(char *, RtToken);
	RtVoid (*CropWindow)(RtFloat, RtFloat, RtFloat, RtFloat);
	RtVoid (*CurveV)(RtToken, RtInt, RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*CylinderV)(RtFloat, RtFloat, RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtToken (*Declare)(char *, char *);
	RtVoid (*DeformationV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*DepthOfField)(RtFloat, RtFloat, RtFloat);
	RtVoid (*Detail)(RtBound);
	RtVoid (*DetailRange)(RtFloat, RtFloat, RtFloat, RtFloat);
	RtVoid (*DiskV)(RtFloat, RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtVoid (*DisplacementV)(RtToken, RtInt, RtToken *, RtPointer *);
	RtToken (*DisplayV)(char *, RtToken, RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*DrawingV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*ErrorHandler)(RtVoid (*)(RtInt, RtInt, char *, RtToken, RtToken));
	RtVoid (*Exposure)(RtFloat, RtFloat);
	RtVoid (*ExteriorV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*Format)(RtInt, RtInt, RtFloat);
	RtVoid (*FrameAspectRatio)(RtFloat);
	RtVoid (*FrameBegin)(RtInt);
	RtVoid (*FrameEnd)(void);
	RtVoid (*FreeHandle)(RtToken, RtToken);
	RtVoid (*GeneralPolygonV)(RtInt, RtInt [], RtInt, RtToken [], RtPointer []);
	RtVoid (*GeometricApproximationV)(RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*GeometricRepresentation)(RtToken);
	RtVoid (*GeometryV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*HiderV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*HyperboloidV)(RtPoint, RtPoint, RtFloat, RtInt, RtToken [], RtPointer []);
	RtVoid (*Identity)(void);
	RtVoid (*Illuminate)(RtToken, RtBoolean);
	RtVoid (*ImagerV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*InteriorV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtToken (*LightSourceV)(char *, RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*LineV)(RtInt, RtInt, RtToken *, RtPointer *);
	RtVoid (*LineStyle)(RtFloat, RtInt, RtFloat [], RtToken, RtToken);
	RtToken (*MacroBeginV)(char *, RtInt, RtToken *, RtPointer *);
	RtVoid (*MacroEnd)(void);
	RtVoid (*MacroInstanceV)(RtToken, RtInt, RtToken *, RtPointer *);
	RtToken (*MakeBumpV)(RtToken, char *, RtToken, RtToken, RtFloatFunc, RtFloat, RtFloat, RtInt, RtToken *, RtPointer *);
	RtToken (*MakeCubeFaceEnvironmentV)(RtToken, RtToken, RtToken, RtToken, RtToken, RtToken, char *, RtFloat, RtFloatFunc, RtFloat, RtFloat, RtInt, RtToken *, RtPointer *);
	RtToken (*MakeLatLongEnvironmentV)(RtToken, char *, RtFloatFunc, RtFloat, RtFloat, RtInt, RtToken *, RtPointer *);
	RtToken (*MakeShadowV)(RtToken, char *, RtInt, RtToken *, RtPointer *);
	RtToken (*MakeTextureV)(RtToken, char *, RtToken, RtToken, RtFloatFunc, RtFloat, RtFloat, RtInt, RtToken *, RtPointer *);
	RtVoid (*Matte)(RtBoolean);
	RtVoid (*MotionBeginV)(RtInt, RtFloat []);
	RtVoid (*MotionEnd)(void);
	RtVoid (*NuCurveV)(RtInt, RtInt, RtFloat *, RtFloat, RtFloat, RtInt, RtToken *, RtPointer *);
	RtVoid (*NuPatchV)(RtInt, RtInt, RtFloat [], RtFloat, RtFloat, RtInt, RtInt, RtFloat [], RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtToken (*ObjectBegin)(RtToken);
	RtVoid (*ObjectEnd)(void);
	RtVoid (*ObjectInstance)(RtToken);
	RtVoid (*Opacity)(RtColor);
	RtVoid (*OptionV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*Orientation)(RtToken);
	RtVoid (*ParaboloidV)(RtFloat, RtFloat, RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtVoid (*PatchMeshV)(RtToken, RtInt, RtToken, RtInt, RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*PatchV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*Perspective)(RtFloat);
	RtVoid (*PixelFilter)(RtFloatFunc, RtFloat, RtFloat);
	RtVoid (*PixelSamples)(RtFloat, RtFloat);
	RtVoid (*PixelVariance)(RtFloat);
	RtVoid (*PointsGeneralPolygonsV)(RtInt, RtInt [], RtInt [], RtInt [], RtInt, RtToken [], RtPointer []);
	RtVoid (*PointsLinesV)(RtInt, RtInt *, RtInt *, RtInt, RtToken *, RtPointer *);
	RtVoid (*PointsPolygonsV)(RtInt, RtInt [], RtInt [], RtInt, RtToken [], RtPointer []);
	RtVoid (*PolygonV)(RtInt, RtInt, RtToken *, RtPointer *);
	RtVoid (*PreemptBegin)(void);
	RtVoid (*PreemptEnd)(void);
	RtVoid (*ProjectionV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*Quantize)(RtToken, RtInt, RtInt, RtInt, RtFloat);
	RtVoid (*ReadArchiveV)(char *, RtVoid (*)(RtToken, char *, char *), RtInt, RtToken *, RtPointer *);
	RtVoid (*RelativeDetail)(RtFloat);
	RtToken (*ResourceV)(char *, RtToken, RtInt, RtToken *, RtPointer *);
	RtVoid (*ReverseOrientation)(void);
	RtVoid (*Rotate)(RtFloat, RtFloat, RtFloat, RtFloat);
	RtVoid (*Scale)(RtFloat, RtFloat, RtFloat);
	RtVoid (*ScreenWindow)(RtFloat, RtFloat, RtFloat, RtFloat);
	RtVoid (*ShadingInterpolation)(RtToken);
	RtVoid (*ShadingRate)(RtFloat);
	RtVoid (*Shutter)(RtFloat, RtFloat);
	RtVoid (*Sides)(RtInt);
	RtVoid (*Skew)(RtFloat, RtFloat, RtFloat, RtFloat, RtFloat, RtFloat, RtFloat);
	RtVoid (*SolidBegin)(RtToken);
	RtVoid (*SolidEnd)(void);
	RtVoid (*SphereV)(RtFloat, RtFloat, RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtVoid (*SurfaceV)(RtToken, RtInt, RtToken [], RtPointer []);
	RtVoid (*Synchronize)(RtToken);
	RtVoid (*TextureCoordinates)(RtFloat, RtFloat, RtFloat, RtFloat, RtFloat, RtFloat, RtFloat, RtFloat);
	RtVoid (*TorusV)(RtFloat, RtFloat, RtFloat, RtFloat, RtFloat, RtInt, RtToken [], RtPointer []);
	RtVoid (*Transform)(RtMatrix);
	RtVoid (*TransformBegin)(void);
	RtVoid (*TransformEnd)(void);
	RtVoid (*Translate)(RtFloat, RtFloat, RtFloat);
	RtVoid (*TrimCurve)(RtInt, RtInt [], RtInt [], RtFloat [], RtFloat [], RtFloat [], RtInt [], RtFloat [], RtFloat [], RtFloat []);
	RtVoid (*WorldBegin)(void);
	RtVoid (*WorldEnd)(void);
} RtRIBHandlers;

RtVoid	QRMGetRIBHandlers(RtRIBHandlers *h),
	QRMSetRIBHandlers(RtRIBHandlers *h);

#endif /* _H_qrman */
